-------------------------------------------------------------------------------
--|> Utils
-- Utilities for common functions.
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local string_format = string.format
local math_floor = math.floor
local math_abs = math.abs
local math_min = math.min
local math_max = math.max
local unpack = unpack
local SELECTED_CHAT_FRAME = SELECTED_CHAT_FRAME
local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
local C_Timer = C_Timer

-------------------------------------------------------------------------------
--|> DISPLAY FUNCTIONS
-------------------------------------------------------------------------------
--> SCALE COORDS
-------------------------------------------------------
--: IN: X,Y coords of where to place something
--: OUT: X, Y coords based effective scale of the game window.
-------------------------------------------------------
function weizPVP.ScaleCoords(xPixel, yPixel)
    local xFactor = weizPVP.Display.xFactor
    local yFactor = weizPVP.Display.yFactor
    local x, y = xPixel / xFactor, yPixel / yFactor
    x, y = x - x % 1, y - y % 1
    return x * xFactor, y * yFactor
end

-------------------------------------------------------------------------------
--|> COLOR FUNCTIONS
-------------------------------------------------------------------------------
--> RGB TO HEX
-----------------------------------------------------------
function weizPVP:RGBToHex(r, g, b)
    r = r <= 255 and r >= 0 and r or 0
    g = g <= 255 and g >= 0 and g or 0
    b = b <= 255 and b >= 0 and b or 0
    return string_format("%02x%02x%02x", r, g, b)
end

--> Adjust RGB Tint
-----------------------------------------------------------
function weizPVP:AdjustRGBTint(rgb, value)
    local r, g, b = unpack(rgb)
    local valueAbs = math_abs(value)
    local h, s, v = weizPVP.RGBToHSV(r, g, b)
    local cv = v
    if value > 0 then
        cv = math_max((cv + value), 0.8)
    else
        cv = math_min((cv + value), 1 - valueAbs)
    end

    local cr, cg, cb = weizPVP.HSVToRGB(h, s, cv)

    return cr, cg, cb
end

--> RGB TO HSV
-----------------------------------------------------------
function weizPVP.RGBToHSV(r, g, b)
    local mincolor, maxcolor = math_min(r, g, b), math_max(r, g, b)
    local ch, cs, cv = 0, 0, maxcolor
    if maxcolor > 0 then
        local delta = maxcolor - mincolor
        cs = delta / maxcolor
        if delta > 0 then
            if r == maxcolor then
                ch = (g - b) / delta
            elseif g == maxcolor then
                ch = 2 + ((b - r) / delta)
            else
                ch = 4 + ((r - g) / delta)
            end
        end
        if ch < 0 then
            ch = ch + 6
        end
        ch = ch / 6
    end
    return ch, cs, cv
end

--> HSV TO RGB
-----------------------------------------------------------
function weizPVP.HSVToRGB(ch, cs, cv)
    if not ch or not cs or not cv then
        return 1, 0, 0
    end
    if ch == 1 then
        ch = 0
    end
    local r, g, b = cv, cv, cv
    if cs > 0 then
        local h = ch * 6
        local sextant = math_floor(h)
        local fract = h - sextant
        local p, q, t = cv * (1 - cs), cv * (1 - (cs * fract)), cv * (1 - (cs * (1 - fract)))
        if sextant == 0 then
            r, g, b = cv, t, p
        elseif sextant == 1 then
            r, g, b = q, cv, p
        elseif sextant == 2 then
            r, g, b = p, cv, t
        elseif sextant == 3 then
            r, g, b = p, q, cv
        elseif sextant == 4 then
            r, g, b = t, p, cv
        else
            r, g, b = cv, p, q
        end
    end
    return r, g, b
end

-------------------------------------------------------------------------------
--|> MATH FUNCTIONS
-------------------------------------------------------------------------------
--> ROUND
-----------------------------------------------------------
function weizPVP:Round(number, decimal)
    local mult = 10 ^ (decimal or 0)
    return math_floor(number * mult + 0.5) / mult
end

-------------------------------------------------------------------------------
--|> CHAT FUNCTIONS
-------------------------------------------------------------------------------
--> PRINT ADDON MESSAGE
-----------------------------------------------------------
--local msgPrefix = "|cffeaeaeaweiz|r|cffffa012PVP|r : "
function weizPVP:PrintAddonMessage(msg)
    --> Print message with default addon prefix
    (SELECTED_CHAT_FRAME or DEFAULT_CHAT_FRAME):AddMessage("|cffeaeaeaweiz|r|cffffa012PVP|r : " .. msg)
end

