-------------------------------------------------------------------------------
--|> CROSSHAIR
--: Places a crosshair on a nameplate.
--: Filtered for player targets that are hostile player with support for icons, text etc attachments
-------------------------------------------------------------------------------
--|> INIT
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local Crosshair = weizPVP:NewModule("WPVPCrosshairFrame", "AceTimer-3.0", "AceEvent-3.0")
weizPVP.Crosshair = Crosshair

-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local IsItemInRange = IsItemInRange
local C_NamePlate = C_NamePlate
local UnitIsPlayer = UnitIsPlayer
local UnitIsEnemy = UnitIsEnemy
local UnitSelectionColor = UnitSelectionColor
local UnitClass = UnitClass
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local IsFlying = IsFlying
local IsUsableItem = IsUsableItem
local UnitIsUnit = UnitIsUnit
local GetTime = GetTime
local C_Timer_After = C_Timer.After
local CreateFrame = CreateFrame
local AuraUtil = AuraUtil
local UIParent = UIParent
local UIFrameFadeOut = UIFrameFadeOut
local UIFrameFadeIn = UIFrameFadeIn
local GetScreenWidth = GetScreenWidth
local GetScreenHeight = GetScreenHeight

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
local TargetNameplate = {}
local lineAlpha = 0.4
weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
local EffectiveScale = weizPVP.Display.EffectiveScale
Crosshair.FrameEffectiveScale = UIParent:GetEffectiveScale()
local NameplateEffectiveScale = UIParent:GetEffectiveScale()

-------------------------------------------------------------------------------
--|> PLUGIN CORE FUNCTIONS
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--|> CREATE CORE FRAME
-------------------------------------------------------------------------------
--> CROSSHAIR FRAME
-------------------------------------------------------
Crosshair.Frame = CreateFrame("Frame", "weizPVP.Crosshair.Frame", UIParent)
Crosshair.Frame:SetFrameStrata("BACKGROUND")
Crosshair.Frame:SetPoint("CENTER")
Crosshair.Frame:SetFrameLevel(0)
Crosshair.Frame:SetSize(64, 64)
Crosshair.Frame:Hide()
local RingCore
local RingFire
local RingFire360Group
local RingFireRotation
local RingInnerShadow
local FourArrows
local FourArrowsAniGTarget1
local ArrowScaleIN
local FourArrowsAniGTarget2
local ArrowAlphaIN
local iconNetOMatic
local lineOffset = 2
local lineThickness = 2
local lineLength = GetScreenWidth()
local lineHeight = GetScreenHeight()
local topLine
local bottomLine
local leftLine
local rightLine
local NameText
local GuildText

--> TARGET RANGE CHECK
-------------------------------------------------------
local function TargetRangeCheck()
	if weizPVP.Options.Crosshair.Enabled == false then
		return
	end
	if not TargetNameplate and Crosshair.Frame:GetAlpha() < 0 then
		UIFrameFadeOut(Crosshair.Frame, 0.1, Crosshair.Frame:GetAlpha(), 0)
		return
	end
	if TargetNameplate then
		--> Get if targetplate exists
		-------------------------------------------------------
		--> Check if Net-o-Matic can be used on target
		if IsFlying("target") and IsItemInRange("Net-o-Matic 5000", "target") and IsUsableItem("Net-o-Matic 5000") then
			iconNetOMatic:SetAlpha(1)
		else
			iconNetOMatic:SetAlpha(0)
		end
	end
	C_Timer_After(0.5, TargetRangeCheck)
end

--> ENABLE CROSSHAIR
-------------------------------------------------------
function weizPVP:EnableCrosshair()
	if weizPVP.Options.Crosshair.Enabled == false then
		return
	end
	--> Register Events
	Crosshair.Frame:RegisterEvent("PLAYER_TARGET_CHANGED")
	Crosshair.Frame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
	Crosshair.Frame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
	Crosshair.Frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	Crosshair.Frame:SetAlpha(0)
	Crosshair.Frame:Show()
	--> Check current target
	Crosshair.Frame:PLAYER_TARGET_CHANGED()
	weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
	Crosshair.FrameEffectiveScale = Crosshair.Frame:GetEffectiveScale()
	EffectiveScale = weizPVP.Display.EffectiveScale
	lineLength = GetScreenWidth()
	lineHeight = GetScreenHeight()
	NameplateEffectiveScale = UIParent:GetEffectiveScale()
	if TargetNameplate then
		NameplateEffectiveScale = TargetNameplate:GetEffectiveScale()
	else
		NameplateEffectiveScale = UIParent:GetEffectiveScale()
	end
	C_Timer_After(0.25, TargetRangeCheck)
end

function weizPVP:CreateCrosshair()
	-------------------------------------------------------------------------------
	--|> CREATE TEXTURES
	-------------------------------------------------------------------------------
	--> Center Ring
	-------------------------------------------------------
	RingCore = Crosshair.Frame:CreateTexture("weizPVP.CrosshairRing", "BACKGROUND", Crosshair.Frame)
	RingCore:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/ring_core_1.tga]])
	RingCore:SetPoint("CENTER", Crosshair.Frame, "CENTER")
	RingCore:SetWidth(128)
	RingCore:SetHeight(128)
	RingCore:SetDrawLayer("ARTWORK", 1)
	RingCore:SetScale(EffectiveScale)
	RingCore:SetBlendMode("ADD")
	--> Ring Shine
	-----------------------------------------------------
	RingFire = Crosshair.Frame:CreateTexture("Crosshair.Frame", "LOW", Crosshair.Frame)
	RingFire:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/ring_fire.tga]])
	RingFire:SetPoint("CENTER", Crosshair.Frame, "CENTER")
	RingFire:SetVertexColor(1, 1, 1)
	RingFire:SetWidth(128)
	RingFire:SetHeight(128)
	RingFire:SetScale(EffectiveScale)
	RingFire:SetAlpha(weizPVP.Options.Crosshair.Alpha)
	RingFire:SetBlendMode("ADD")
	RingFire:SetDrawLayer("ARTWORK", 7)
	RingFire:Hide()
	--> Circle Shine Animation: Rotate
	-------------------------------------------------------
	RingFire360Group = RingFire:CreateAnimationGroup()
	RingFireRotation = RingFire360Group:CreateAnimation("Rotation")
	RingFireRotation:SetDegrees(-360)
	RingFireRotation:SetDuration(2)
	RingFire360Group:SetLooping("REPEAT")
	RingFire360Group:Play()
	--> Background
	-------------------------------------------------------
	RingInnerShadow = Crosshair.Frame:CreateTexture("Crosshair.RingInnerShadow", "LOW", Crosshair.Frame)
	RingInnerShadow:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/ring_innerShadow.tga]])
	RingInnerShadow:SetPoint("CENTER", Crosshair.Frame, "CENTER")
	RingInnerShadow:SetWidth(128)
	RingInnerShadow:SetHeight(128)
	RingInnerShadow:SetVertexColor(0, 0, 0, 0.5)
	RingInnerShadow:SetScale(EffectiveScale)
	RingInnerShadow:SetAlpha(weizPVP.Options.Crosshair.Alpha)
	RingInnerShadow:SetBlendMode("BLEND")
	RingInnerShadow:SetDrawLayer("BACKGROUND", 1)

	-------------------------------------------------------------------------------
	--|>|>  Four Arrows  <|<|--
	-------------------------------------------------------------------------------
	--> Four Arrows Texture
	-------------------------------------------------------
	FourArrows = Crosshair.Frame:CreateTexture("weizPVP.Crosshair.Frame.FourArrows", "LOW")
	FourArrows:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/4Arrows.tga]])
	FourArrows:SetPoint("CENTER", Crosshair.Frame, "CENTER")
	FourArrows:SetWidth(96)
	FourArrows:SetHeight(96)
	FourArrows:SetDrawLayer("ARTWORK", 1)
	FourArrows:SetVertexColor(1, 0, 0)
	FourArrows:SetScale(1)
	FourArrows:SetAlpha(weizPVP.Options.Crosshair.Alpha)
	FourArrows:SetBlendMode("ADD")
	--> FourArrows: Transition to Target Animation
	-------------------------------------------------------
	-->> Scale In
	FourArrowsAniGTarget1 = FourArrows:CreateAnimationGroup()
	ArrowScaleIN = FourArrowsAniGTarget1:CreateAnimation("Scale")
	ArrowScaleIN:SetDuration(0.4)
	ArrowScaleIN:SetFromScale(3, 3)
	ArrowScaleIN:SetToScale(1, 1)
	ArrowScaleIN:SetSmoothing("OUT")
	-->> Fade In
	FourArrowsAniGTarget2 = FourArrows:CreateAnimationGroup()
	ArrowAlphaIN = FourArrowsAniGTarget2:CreateAnimation("Alpha")
	ArrowAlphaIN:SetDuration(0.4)
	ArrowAlphaIN:SetFromAlpha(0)
	ArrowAlphaIN:SetToAlpha(weizPVP.Options.Crosshair.Alpha)

	-------------------------------------------------------------------------------
	--|> CREATE LINES
	-------------------------------------------------------------------------------
	local function CreateLineTemplate(CorePointAncho, StartLinePoint, OffsetEndX, OffsetEndY)
		local f = Crosshair.Frame:CreateLine("weizPVP.Crosshairs.leftLine", "LOW")
		f:SetTexture([[interface/addons/weizPVP/Media/Textures/white16x16.tga]])
		f:SetThickness(lineThickness)
		f:SetStartPoint(CorePointAncho, Crosshair.Frame, StartLinePoint, -lineOffset, 0)
		f:SetEndPoint(CorePointAncho, OffsetEndX, OffsetEndY)
		f:SetVertexColor(1, 1, 1)
		f:SetAlpha(lineAlpha)
		f:SetBlendMode("ADD")
		return f
	end
	-->> LEFT LINE
	-------------------------------------------------------
	leftLine = CreateLineTemplate("LEFT", "RIGHT", -lineLength, 0)
	rightLine = CreateLineTemplate("RIGHT", "LEFT", lineLength, 0)
	topLine = CreateLineTemplate("TOP", "BOTTOM", 0, lineHeight)
	bottomLine = CreateLineTemplate("BOTTOM", "TOP", 0, -lineHeight)
	-------------------------------------------------------------------------------
	--|> CREATE ADDITIONAL INDICATORS
	-------------------------------------------------------------------------------
	--> Net-O-Matic Icon
	---------------------------------------------------------------------
	iconNetOMatic = Crosshair.Frame:CreateTexture(nil, "MEDIUM")
	iconNetOMatic:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/netomatic.tga]])
	iconNetOMatic:SetSize(24, 24)
	iconNetOMatic:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", 0, 12)
	iconNetOMatic:SetAlpha(0)

	--> Target NameText
	---------------------------------------------------------------------
	NameText = Crosshair.Frame:CreateFontString(nil, "DIALOG", nil, 1)
	NameText:SetFont([[interface/addons/weizPVP/Media/Fonts/RobotoCondensed-BoldItalic.ttf]], 16, "NONE")
	NameText:SetHeight(16)
	NameText:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", weizPVP.ScaleCoords(16, 4))
	NameText:SetJustifyH("LEFT")
	NameText:SetText("")
	NameText:SetTextColor(1, 1, 1)
	NameText:SetShadowColor(0, 0, 0, 1)
	NameText:SetShadowOffset(-1, 1)
	NameText:SetAlpha(1)
	-- if not weizPVP.ReplaceTargetedNameplate then
	NameText:Hide()
	-- end

	--> Target Name Text Background Text
	-------------------------------------------------------
	NameText.BG = Crosshair.Frame:CreateFontString(nil, "BACKGROUND", nil, 1)
	NameText.BG:SetFont([[interface/addons/weizPVP/Media/Fonts/RobotoCondensed-BoldItalic.ttf]], 16, "OUTLINE")
	NameText.BG:SetPoint("CENTER", NameText, "CENTER", weizPVP.ScaleCoords(2, -1))
	NameText.BG:SetDrawLayer("BACKGROUND", 1)
	NameText.BG:SetHeight(NameText:GetHeight())
	NameText.BG:SetText(NameText:GetText())
	NameText.BG:SetJustifyH(NameText:GetJustifyH())
	NameText.BG:SetTextColor(0, 0, 0)
	NameText.BG:SetShadowColor(0, 0, 0, 1)
	NameText.BG:SetShadowOffset(1, -1)
	NameText.BG:SetAlpha(1)
	NameText.BG:Hide()

	--> Target Name: BG GRAD
	-------------------------------------------------------
	NameText.GradBG = Crosshair.Frame:CreateTexture(nil, "BACKGROUND")
	NameText.GradBG:SetHeight(12)
	NameText.GradBG:SetPoint("LEFT", rightLine, "LEFT", 0, 0)
	NameText.GradBG:SetPoint("RIGHT", NameText, "RIGHT", weizPVP.ScaleCoords(12, 0))
	NameText.GradBG:SetPoint("BOTTOM", rightLine, "TOP", weizPVP.ScaleCoords(0, 1))
	NameText.GradBG:SetTexture(1, 0, 0, 1)
	NameText.GradBG:SetGradientAlpha("Vertical", 0, 0, 0, 0.4, 0, 0, 0, 0)

	--> Target Name: BG GRAD END CORNER
	-------------------------------------------------------
	NameText.GradBGCorner = Crosshair.Frame:CreateTexture(nil, "BACKGROUND")
	NameText.GradBGCorner:SetHeight(NameText.GradBG:GetHeight())
	NameText.GradBGCorner:SetWidth(NameText.GradBG:GetHeight())
	NameText.GradBGCorner:SetPoint("TOPLEFT", NameText.NameTextGradBG, "TOPRIGHT", weizPVP.ScaleCoords(0, 0))
	NameText.GradBGCorner:SetTexture([[interface/addons/weizPVP/Media/Textures/cornerGrad.tga]])
	NameText.GradBGCorner:SetVertexColor(0, 0, 0, 0.4)
	NameText.GradBGCorner:SetAlpha(0.4)
	NameText.GradBGCorner:SetRotation(1.570795)
	NameText.GradBGCorner:SetBlendMode("BLEND")

	--> Target: Guild Text
	---------------------------------------------------------------------
	GuildText = Crosshair.Frame:CreateFontString(nil, "ARTWORK", nil, 1)
	GuildText:SetFont([[interface/addons/weizPVP/Media/Fonts/Accidental-Presidency-Italic.ttf]], 12, "NONE")
	GuildText:SetHeight(12)
	GuildText:SetPoint("TOPLEFT", NameText, "BOTTOMLEFT", weizPVP.ScaleCoords(0, -4))
	GuildText:SetJustifyH("LEFT")
	GuildText:SetText("")
	GuildText:SetTextColor(.9, .9, .9)
	GuildText:SetShadowColor(0, 0, 0, 1)
	GuildText:SetShadowOffset(-1, 1)
	GuildText:SetAlpha(1)
	GuildText:Hide()

	--> Guild Text: BG
	-------------------------------------------------------
	GuildText.BG = Crosshair.Frame:CreateFontString(nil, "BACKGROUND", nil, 1)
	GuildText.BG:SetFont([[interface/addons/weizPVP/Media/Fonts/Accidental-Presidency-Italic.ttf]], GuildText:GetHeight(), "NONE")
	GuildText.BG:SetPoint("CENTER", GuildText, "CENTER", weizPVP.ScaleCoords(1, -1))
	GuildText.BG:SetDrawLayer("BACKGROUND", 6)
	GuildText.BG:SetHeight(GuildText:GetHeight())
	GuildText.BG:SetText(GuildText:GetText())
	GuildText.BG:SetJustifyH(GuildText:GetJustifyH())
	GuildText.BG:SetTextColor(0.1, 0.1, 0.1)
	GuildText.BG:SetShadowColor(0, 0, 0, 1)
	GuildText.BG:SetShadowOffset(1, -1)
	GuildText.BG:SetAlpha(1)
	GuildText.BG:Hide()

	--> Guild Text: BG GRAD
	-------------------------------------------------------
	GuildText.GradBG = Crosshair.Frame:CreateTexture(nil, "BACKGROUND")
	GuildText.GradBG:SetHeight(14)
	GuildText.GradBG:SetPoint("LEFT", rightLine, "LEFT", 0, 0)
	GuildText.GradBG:SetPoint("RIGHT", GuildText, "RIGHT", weizPVP.ScaleCoords(12, 0))
	GuildText.GradBG:SetPoint("TOP", rightLine, "TOP", weizPVP.ScaleCoords(0, -1))
	GuildText.GradBG:SetTexture(1, 0, 0, 1)
	GuildText.GradBG:SetGradientAlpha("Vertical", 0, 0, 0, 0, 0, 0, 0, 0.4)
	GuildText.GradBG:SetDrawLayer("BACKGROUND", -7)

	--> Guild Text: BG GRAD END CORNER
	-------------------------------------------------------
	GuildText.GradBGCorner = Crosshair.Frame:CreateTexture(nil, "BACKGROUND")
	GuildText.GradBGCorner:SetHeight(GuildText.GradBG:GetHeight())
	GuildText.GradBGCorner:SetWidth(GuildText.GradBG:GetHeight())
	GuildText.GradBGCorner:SetPoint("TOPLEFT", GuildText.GradBG, "TOPRIGHT", weizPVP.ScaleCoords(0, 0))
	GuildText.GradBGCorner:SetTexture([[interface/addons/weizPVP/Media/Textures/cornerGrad.tga]])
	GuildText.GradBGCorner:SetVertexColor(0, 0, 0, 0.4)
	GuildText.GradBGCorner:SetAlpha(0.4)
	GuildText.GradBGCorner:SetBlendMode("BLEND")
	GuildText.GradBGCorner:SetDrawLayer("BACKGROUND", -7)
end

-------------------------------------------------------------------------------
--|> UI FUNCTIONS
-------------------------------------------------------------------------------
--> Set Color of lines and circle
-------------------------------------------------------
local CurrentClassColor = {1, 1, 1}
local function SetColor(r, g, b)
	CurrentClassColor = {r, g, b}
	--> TEXTURES
	NameText:SetTextColor(weizPVP:AdjustRGBTint({r, g, b}, 0.3))
	NameText.BG:SetTextColor(weizPVP:AdjustRGBTint({r, g, b}, -0.6))
	--> RINGS
	RingCore:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, 0.2))
	--> Lines
	topLine:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, -0.1))
	bottomLine:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, -0.1))
	leftLine:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, -0.1))
	rightLine:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, -0.1))
end
--> DISABLE CROSSHAIR
-------------------------------------------------------
function weizPVP:DisableCrosshair()
	--> Clear plate and fade out
	TargetNameplate = nil
	UIFrameFadeOut(Crosshair.Frame, 0.1, Crosshair.Frame:GetAlpha(), 0)
	Crosshair.Frame:Hide()
	--> Unregister Events
	Crosshair.Frame:UnregisterEvent("PLAYER_TARGET_CHANGED")
	Crosshair.Frame:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
	Crosshair.Frame:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
	Crosshair.Frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

--> Check for Bounty
-------------------------------------------------------
local function CheckForBounty()
	--> Check for Bounty
	local bounty = AuraUtil.FindAuraByName("Bounty Hunted", "target", "HARMFUL")
	if bounty == "Bounty Hunted" then
		RingFire:Show()
	else
		RingFire:Hide()
	end
end

--> Get Nameplate
-------------------------------------------------------
local Moving = GetTime()
local x
local y
local fScale
local class
local classColor
local r, g, b

local function GetNameplate(plate)
	--> Set on same level as nameplate
	Crosshair.Frame:SetParent(plate)
	x, y = nil, nil
	if Crosshair.Frame:GetAlpha() == 0 then
		x, y = plate:GetCenter()
		if x and y then
			x, y = x * Crosshair.FrameEffectiveScale, y * Crosshair.FrameEffectiveScale
			fScale = Crosshair.Frame:GetScale()
			x, y = x / fScale, y / fScale
			Crosshair.Frame:SetPoint("CENTER", plate)
		end
		UIFrameFadeIn(Crosshair.Frame, 0.3, 0, weizPVP.Options.Crosshair.Alpha)
	else
		Crosshair.Frame:SetAlpha(weizPVP.Options.Crosshair.Alpha)
	end
	if FourArrowsAniGTarget1:IsPlaying() then
		FourArrowsAniGTarget1:Stop()
	end
	if FourArrowsAniGTarget2:IsPlaying() then
		FourArrowsAniGTarget2:Stop()
	end
	FourArrowsAniGTarget1:Play()
	FourArrowsAniGTarget2:Play()

	--> Bounty Check
	if weizPVP.Options.Crosshair.ShowBountyOverlay == true then
		CheckForBounty()
	else
		RingFire:Hide()
	end

	--> Set Class Colors
	r, g, b = UnitSelectionColor("target")
	if UnitIsPlayer("target") then
		class = select(2, UnitClass("target"))
		classColor = RAID_CLASS_COLORS[class]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		end
	end
	--> APPLY COLORS
	SetColor(r, g, b)
	Moving = GetTime()
end
--> FUNCTION: TargetRangeCheck
-------------------------------------------------------
local x1, y1, x2, y2, d1, d2, amountToMove, ratio
local distance, timeLeft
local nameplate

-------------------------------------------------------------------------------
--|> EVENTS
-------------------------------------------------------------------------------

--> FUNCTION: Nameplate Retry
--: Re-checks to see if the target has a nameplate we can attach to...
--: Sometimes there seems to be a delay between getting a target, and registering the nameplate
local NameplateGetAttempts = 0
local lockPos = false
local retrying = false
local function NameplateRetry()
	nameplate = C_NamePlate.GetNamePlateForUnit("target")
	if nameplate then
		Crosshair.Frame:PLAYER_TARGET_CHANGED()
		lockPos = false
	else
		--> Check for valid nameplate existing
		if NameplateGetAttempts < 10 then
			NameplateGetAttempts = NameplateGetAttempts + 1
			C_Timer_After(0.1, NameplateRetry)
		else
			TargetNameplate = nil
		end
	end
end

-------------------------------------------------------
--=> PLAYER_TARGET_CHANGED
-------------------------------------------------------
function Crosshair.Frame:PLAYER_TARGET_CHANGED()
	local targetValid = true
	--> Check for a player
	if not UnitIsPlayer("target") then
		targetValid = false
	end
	--> Check for enemy
	if not (UnitIsEnemy("player", "target") or weizPVP.Options.DEBUG == true) then
		targetValid = false
	end
	--> Activate
	if targetValid then
		nameplate = C_NamePlate.GetNamePlateForUnit("target")
		if not nameplate then
			if lockPos == false then
				NameplateGetAttempts = 0
				if not retrying then
					C_Timer_After(0.1, NameplateRetry)
					RingFire:Hide()
				end
				lockPos = true
				retrying = true
				UIFrameFadeOut(Crosshair.Frame, 0.1, Crosshair.Frame:GetAlpha(), 0)
				TargetNameplate = nil
			end
			return
		end
		if Crosshair.Frame:GetAlpha() < weizPVP.Options.Crosshair.Alpha then
			UIFrameFadeIn(Crosshair.Frame, 0.1, 0, weizPVP.Options.Crosshair.Alpha)
		end
		lockPos = false
		TargetNameplate = nameplate
		GetNameplate(nameplate)

		--> Adds Nameplate elements if enabled
		if weizPVP.Options.Crosshair.NameEnabled then
			NameText:Show()
			NameText:SetAlpha(1)
			NameText.BG:SetAlpha(1)
			NameText.GradBG:Show()
			NameText.GradBGCorner:Show()
			--> NAME TEXT
			local name = GetUnitName("target", true)
			local charname = gsub(name, "-(.*)", "")
			if strlen(name) > strlen(charname) then
				NameText:SetText(charname .. "|cFFFF00CC*|r")
			else
				NameText:SetText(name)
			end
			NameText.BG:SetText(NameText:GetText())
		else
			NameText:SetAlpha(0)
			NameText.BG:SetAlpha(0)
			NameText.GradBG:Hide()
			NameText.GradBGCorner:Hide()
		end

		if weizPVP.Options.Crosshair.GuildEnabled then
			GuildText:SetAlpha(1)
			GuildText:Show()
			GuildText.BG:SetAlpha(1)
			GuildText.GradBG:Show()
			GuildText.GradBGCorner:Show()
			GuildText:SetText(GetGuildInfo("target") or "")
			GuildText.BG:SetText(GuildText:GetText())
			if not GuildText:GetText() or GuildText:GetText() == "" then
				GuildText.GradBG:Hide()
			else
				GuildText.GradBG:Show()
			end
		else
			GuildText:SetAlpha(0)
			GuildText.BG:SetAlpha(0)
			GuildText.GradBG:Hide()
			GuildText.GradBGCorner:Hide()
		end
		NameplateEffectiveScale = EffectiveScale
		Crosshair.FrameEffectiveScale = EffectiveScale
	else --> Fade out if not an enemy or no target
		if Crosshair.Frame:GetAlpha() ~= 0 then
			UIFrameFadeOut(Crosshair.Frame, 0.1, weizPVP.Options.Crosshair.Alpha, 0)
		end
		TargetNameplate = nil
	end
end

-------------------------------------------------------
--=> EVENT: PLAYER_ENTERING_WORLD
-------------------------------------------------------
function Crosshair.Frame:PLAYER_ENTERING_WORLD()
	Crosshair.Frame:PLAYER_TARGET_CHANGED()
end

--> CROSSHAIR: MOVE
-------------------------------------------------------
local function MoveCrosshair()
	x1, y1 = Crosshair.Frame:GetCenter()
	x1, y1 = x1 * Crosshair.FrameEffectiveScale, y1 * Crosshair.FrameEffectiveScale
	x2, y2 = TargetNameplate:GetCenter()
	x2, y2 = x2 * NameplateEffectiveScale, y2 * NameplateEffectiveScale
	d1, d2 = y2 - y1, x2 - x1
	distance = (d2 ^ 2 + d1 ^ 2) ^ 0.5
	timeLeft = Moving + 1 - GetTime()
	--> Still moving towards the new/current nameplate from last plate's locattion
	if timeLeft > .1 and distance > 2 then
		amountToMove = distance / timeLeft / (weizPVP.Display.CurrentFPS / 10)
		ratio = amountToMove / distance
		x = x1 + ratio * d2
		y = y1 + ratio * d1
		Crosshair.Frame:SetPoint("CENTER", TargetNameplate, "CENTER", weizPVP.ScaleCoords(x - x2, y - y2))
	else --> Crosshairs are now anchored to the unit's nameplate
		Crosshair.Frame:SetPoint("CENTER", TargetNameplate, "CENTER")
		-- Attempt to lower the level of the nameplate so that it is behind the nameplate or nameplate addons.
		Crosshair.Frame:SetFrameLevel(tonumber(TargetNameplate:GetFrameLevel()) - 1)
		Moving = false
	end
	return
end

--> NAME_PLATE_UNIT_ADDED
-------------------------------------------------------
local nameplate_added
function Crosshair.Frame:NAME_PLATE_UNIT_ADDED(unit)
	if UnitIsUnit("target", unit) then
		nameplate_added = C_NamePlate.GetNamePlateForUnit("target")
		if nameplate_added and UnitIsPlayer("target") and (UnitIsEnemy("player", "target") or weizPVP.Options.DEBUG) then
			Crosshair.Frame:Show()
			UIFrameFadeIn(Crosshair.Frame, 0.1, 0, weizPVP.Options.Crosshair.Alpha)
			Crosshair.Frame:PLAYER_TARGET_CHANGED()
			-- Attempt to lower the level of the nameplate so that it is behind the nameplate or nameplate addons.
			Crosshair.Frame:SetFrameLevel(tonumber(TargetNameplate:GetFrameLevel()) - 1)
		end
	end
end

--> NAME_PLATE_UNIT_REMOVED
-------------------------------------------------------
function Crosshair.Frame:NAME_PLATE_UNIT_REMOVED(unit)
	if UnitIsUnit("target", unit) then
		TargetNameplate = nil
		if Crosshair.Frame:GetAlpha() > weizPVP.Options.Crosshair.Alpha then
			UIFrameFadeOut(Crosshair.Frame, 0.1, weizPVP.Options.Crosshair.Alpha, 0)
			Crosshair.Frame:Hide()
		end
	end
end

--> ON_EVENT: self
-------------------------------------------------------
Crosshair.Frame:SetScript(
	"OnEvent",
	function(self, event, ...)
		return self[event] and self[event](self, ...)
	end
)

--> ON_UPDATE
-------------------------------------------------------
Crosshair.Frame:SetScript(
	"OnUpdate",
	function()
		if TargetNameplate and Moving and weizPVP.Options.Crosshair.Enabled == true then
			MoveCrosshair()
		end
	end
)

--> CROSSHAIR ALPHA
-------------------------------------------------------
function weizPVP:CrosshairsSetAlpha(value)
	weizPVP.Options.Crosshair.Alpha = value
	Crosshair.Frame:SetAlpha(value)
	RingFire:SetAlpha(value)
	RingInnerShadow:SetAlpha(value)
	FourArrows:SetAlpha(value)
	ArrowAlphaIN:SetToAlpha(value)
	lineAlpha = 0.4 * (1 / value)
	leftLine:SetAlpha(lineAlpha)
	rightLine:SetAlpha(lineAlpha)
	topLine:SetAlpha(lineAlpha)
	bottomLine:SetAlpha(lineAlpha)
end

--> TOGGLE CROSSHAIRS
-------------------------------------------------------
function weizPVP:ToggleCrosshairs()
	if weizPVP.Options.Crosshair.Enabled == false then
		weizPVP.Options.Crosshair.Enabled = true
		weizPVP:EnableCrosshair()
		weizPVP:PrintAddonMessage("Crosshairs are |cff37ff37ENABLED|r.")
	else
		weizPVP.Options.Crosshair.Enabled = false
		weizPVP:DisableCrosshair()
		weizPVP:PrintAddonMessage("Crosshairs are |cffff3838DISABLED|r.")
	end
end
